from kubernetes.client import Configuration
from kubernetes.client.api import core_v1_api
from kubernetes.client.rest import ApiException
from kubernetes.stream import stream
from kubernetes import client,config
from prometheus_api_client import PrometheusConnect
config.load_kube_config()
v1=client.CoreV1Api()

def manage_pods(replicas, docker_image, name_pod, cpu_limits, cpu_request, namespace_pod):
    config.load_kube_config()
    apps_api = client.AppsV1Api()
    deployment = client.V1Deployment()
    deployment.api_version = "apps/v1"
    deployment.kind = "Deployment"
    deployment.metadata = client.V1ObjectMeta(name=name_pod)
    spec = client.V1DeploymentSpec(
        selector=client.V1LabelSelector(match_labels={"app":name_pod}),
        template=client.V1PodTemplateSpec(),
    )
    container = client.V1Container(
        image=docker_image,
        resources = {"limits": {"cpu":cpu_limits} , "requests": {"cpu":cpu_request}},
        name=name_pod, 
    )
    spec.template.metadata = client.V1ObjectMeta(
        name=name_pod,
        labels={"app":name_pod},
    )
    spec.template.spec = client.V1PodSpec(containers = [container])
    dep = client.V1Deployment(
        metadata=client.V1ObjectMeta(name=name_pod),
        spec=spec,
    )
    dep.spec.replicas = replicas
    try:
        apps_api.create_namespaced_deployment(namespace=namespace_pod, body=dep)
    except:
        print("Pod Exist, updating replicas")    
    dep.spec.replicas = replicas
    changeddeploy= apps_api.replace_namespaced_deployment(name=name_pod, namespace=namespace_pod, body=dep)


def get_pod_metrics_scenario_1(server, lapso, namespace, podname, requestpodname):

    prom = PrometheusConnect(url=server, disable_ssl=True)

    strQuery='sum(rate(container_file_descriptors{pod=~"' + podname +'[^v]*", container!="POD", namespace="' + namespace + '", container ="' + podname +'"}['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    try:
        file_descriptors=resultado[0]["value"][1]
    except:
        file_descriptors = "0"
    
    strQuery='sum(rate(container_network_receive_packets_total{pod=~"' + podname +'[^v]*", namespace="' + namespace + '",  interface="eth0"}['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    try:
        receive_packets=resultado[0]["value"][1]
    except:
        receive_packets = "0"

    strQuery='sum(rate(container_network_transmit_packets_total{pod=~"' + podname +'[^v]*", namespace="'+ namespace + '" , interface="eth0"}['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    try:
        transmit_packets=resultado[0]["value"][1]
    except:
        transmit_packets = "0"

    strQuery='sum(rate(container_network_transmit_packets_dropped_total{namespace="'+ namespace +'",pod="'+ requestpodname + '",interface="eth0"}['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    try:
        dropped_packets=resultado[0]["value"][1]
    except:
        dropped_packets = "0"
    

    strQuery='sum(rate(container_cpu_usage_seconds_total{container_name!="POD",container!="POD" ,namespace="'+ namespace +'" ,pod=~"' + podname +'[^v]*", }['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    try:
        cpu_usage_seconds=resultado[0]["value"][1]
    except:
        cpu_usage_seconds = "0"

    strQuery='sum(rate(container_cpu_cfs_throttled_seconds_total{ namespace="'+ namespace + '", pod=~"' + podname +'[^v]*" }['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    
    try:
        cpu_throttled_seconds=resultado[0]["value"][1]
    except:
        cpu_throttled_seconds = "0"    

    strQuery='sum(rate(container_memory_working_set_bytes{ namespace="' + namespace + '", pod=~"' + podname +'[^v]*"}['+lapso+']))'
    resultado = prom.custom_query(query=strQuery)
    
    try:
        memory_working_bytes=resultado[0]["value"][1]
    except:
        memory_working_bytes = "0"    
    
    
    strQuery='count(container_memory_usage_bytes{container="' + podname +'", namespace="'+ namespace +'"})'
    resultado = prom.custom_query(query=strQuery)
    
    try:
        memory_usage_bytes=resultado[0]["value"][1]
    except:
        memory_usage_bytes = "0"    
    
    
    ret=v1.list_namespaced_pod(namespace,watch=False)
    pods=ret.items
    pods_names=[pod.metadata.name for pod in pods]
    number_pods= 0
    for pod in pods_names:
        if (pod.find(podname)>-1):
            number_pods = number_pods+1
    
    formatted_pods = "{:.1f}".format(float(number_pods))
    formatted_file_descriptors = "{:.1f}".format(float(file_descriptors))
    formatted_receive_packets = "{:.1f}".format(float(receive_packets))
    formatted_transmit_packets = "{:.1f}".format(float(transmit_packets))
    formatted_dropped_packets = "{:.1f}".format(float(dropped_packets))
    formatted_cpu_usage_seconds = "{:.10f}".format(float(cpu_usage_seconds))
    formatted_cpu_throttled_seconds = "{:.10f}".format(float(cpu_throttled_seconds))
    formatted_memory_working_bytes = "{:.1f}".format(float(memory_working_bytes))
    formatted_memory_usage_bytes = "{:.1f}".format(float(memory_usage_bytes))

    return{"pods": formatted_pods,"file_descriptors":formatted_file_descriptors,"receive_packets":formatted_receive_packets,
          "transmit_packets":formatted_transmit_packets, "dropped_packets":formatted_dropped_packets,
          "cpu_usage_seconds":formatted_cpu_usage_seconds, "cpu_throttled_seconds": formatted_cpu_throttled_seconds,
          "memory_working_bytes": formatted_memory_working_bytes, "memory_usage_bytes" :formatted_memory_usage_bytes}




import time

from kubernetes import config
from kubernetes.client import Configuration
from kubernetes.client.api import core_v1_api
from kubernetes.client.rest import ApiException
from kubernetes.stream import stream

class Pod:

	def __init__(self, namepod ,dockerimage ,namespace = "default", shell = "/bin/sh"):
	
		"""Function to read in data from a txt file. The txt file should have
		one number (float) per line. The numbers are stored in the data attribute.
				
		Args:
			file_name (string): name of a file to read from
		
		Returns:
			None
		
		"""
			
		self.namepod = namepod
		self.apiversion = "v1"
		self.kind = "Pod"
		self.dockerimage = dockerimage
		self.namespace = namespace        
		self.resp = None
		self.shell = shell

	def create(self):
	
		"""Function to read in data from a txt file. The txt file should have
		one number (float) per line. The numbers are stored in the data attribute.
				
		Args:
			file_name (string): name of a file to read from
		
		Returns:
			None
		
		"""
		config.load_kube_config()
		try:
			c = Configuration().get_default_copy()
		except AttributeError:
			c = Configuration()
			c.assert_hostname = False
		Configuration.set_default(c)
		core_v1 = core_v1_api.CoreV1Api()
		api_instance = core_v1
		try:
			self.resp = api_instance.read_namespaced_pod(name=self.namepod, namespace=self.namespace)
		except ApiException as e:
			if e.status != 404:
				print("Unknown error: %s" % e)
				exit(1)
		if not self.resp:
			print("Pod %s does not exist. Creating it..." + self.namepod)
			pod_manifest = {
				'apiVersion': 'v1',
				'kind': self.kind,
				'metadata': {
					'name': self.namepod
				},
				'spec': {
					'containers': [{
						'image': self.dockerimage,
						'name': 'sleep',
						"args": [
							"/bin/sh",
							"-c",
							"while true;do date;sleep 5; done"
						]
					}]
				}
			}
			self.resp = api_instance.create_namespaced_pod(body=pod_manifest, namespace=self.namespace)
			while True:
				self.resp = api_instance.read_namespaced_pod(name=self.namepod, namespace=self.namespace)
				if self.resp.status.phase != 'Pending':
					break
				time.sleep(1)
			print("Done.")

	def exec_command(self, strexec):

		core_v1 = core_v1_api.CoreV1Api()
		api_instance = core_v1
		print(strexec)
		self.resp = stream(api_instance.connect_get_namespaced_pod_exec, 
			self.namepod,
			self.namespace,
			command = self.shell,
			stderr=True, stdin=True,
			stdout=True, tty=False,
			_preload_content=False)
		self.resp.write_stdin(strexec + "\n")





from setuptools import setup

setup(name='otumba',
      version='0.2.2',
      description='AI on Cloud Distributed Systems',
      packages=['otumba'],
      author = 'Pedro Coronel Santeliz',
      author_email = 'pelsanmexico@gmail.com',
      zip_safe=False)
